#include <Rotary.h>
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_ST7735.h>

#define pulseHigh(pin) {digitalWrite(pin, HIGH); digitalWrite(pin, LOW); }

//set DDS pins
#define W_CLK A2  // AD9850 load clock pin (FCLK)
#define FQ_UD A3   // freq update pin (FQUD)
#define DATA  A4   // serial data load pin (DATA)
#define RESET A5   // reset pin (RST) 

// TFT
#define sclk 13
#define mosi 11
#define cs   5
#define rst  6  
#define dc   7

Adafruit_ST7735 tft = Adafruit_ST7735(cs, dc, rst);

Rotary r = Rotary(2,3); // sets the pins the rotary encoder uses.  
//LedControl lc=LedControl(7,6,5,1);
const int buttonPin = 4;

int buttonState = 0;
int_fast32_t rx = 20000000; // Starting frequency of VFO 14.100 MHz
int_fast32_t rx_old;        // variable to hold the updated frequency
int_fast32_t rx_ofs;        // command frequency for DDS incl. possible IF offset 
int_fast32_t rx2=1; // variable to hold the updated frequency
int_fast32_t step1 = 1000;   // starting VFO update increment in HZ.

float rx_disp;           
int  stp = 2;                           // starting with 100Hz steps 
int  stp_old;

////////////////////////////////////////////////////////////
void setup() {    
  pinMode(buttonPin, INPUT);

  tft.initR(INITR_BLACKTAB);   
  tft.setRotation(tft.getRotation() + 1); // rotate 90degs
  tft.setTextSize(1);    //1-8
  tft.fillScreen(ST7735_BLACK);
  tft.setCursor(20, 2);
  tft.setTextColor(ST7735_GREEN);
  tft.print("DDS Generator AD9850");
  tft.setTextSize(2); 
  tft.setCursor(15, 25);
  tft.setTextColor(ST7735_BLUE);
  tft.print("Freq:");



  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();

  pinMode(FQ_UD, OUTPUT);
  pinMode(W_CLK, OUTPUT);
  pinMode(DATA, OUTPUT);
  pinMode(RESET, OUTPUT); 

  pulseHigh(RESET);
  pulseHigh(W_CLK);
  pulseHigh(FQ_UD);  // this pulse enables serial mode on the AD9850 - Datasheet page 12.
}

////////////////////////////////////////////////////////////  
void loop() {
  read_Step();
  read_Step_Select();
  buttonState = digitalRead(buttonPin);
if (rx != rx2)
{

  rx_disp = rx;
  rx_disp = rx_disp/1000000;
  rx2 = rx;

  tft.fillRect(5,45,155,20,ST7735_BLACK);   //partly screen erasing by writing box
  tft.setCursor(15, 50);
  tft.setTextColor(ST7735_WHITE);
  tft.print(rx_disp,5 );
  tft.print(" MHz");

  rx_old = rx;
  rx_ofs = rx + 0;             // add IF offset frequency if required    
  sendFrequency(rx_ofs);       // frequency for controlling the DDS 
}
}  

////////////////////////////////////////////////////////////
// frequency calc 
void sendFrequency(double frequency) {  
  int32_t freq = frequency * 4294967295/125000000;  // note 125 MHz clock on AD9850.  You can make 'slight' tuning variations here by adjusting the clock frequency.
  for (int b=0; b<4; b++, freq>>=8) {
    tfr_byte(freq & 0xFF);
  }
  tfr_byte(0x000);   // Final control byte, all 0 for AD9850 chip
  pulseHigh(FQ_UD);  // Done!  Should see output
}

////////////////////////////////////////////////////////////
// transfers a byte, a bit at a time, LSB first to the AD9850 via serial DATA line
void tfr_byte(byte data) {
  for (int i=0; i<8; i++, data>>=1) 
  {
    digitalWrite(DATA, data & 0x01);
    pulseHigh(W_CLK);   //after each bit sent, CLK is pulsed high
  }
}

/////////////////////////////////////////////////////////////////
void read_Step_Select() {
  if(stp==7)  
  {
    stp=1; 
  }
  if(stp==0)  
  {
    stp=6; 
  }

  if(stp!=stp_old)  
  {
    if (stp==1) 
    {   
      step1 = 10;
  tft.fillRect(5,100,155,20,ST7735_BLACK);   //partly screen erasing by writing box
  tft.setCursor(15, 80);
  tft.setTextColor(ST7735_CYAN);
  tft.print("Step:");
  tft.setCursor(15, 105);
  tft.setTextColor(ST7735_WHITE);
  tft.print("10 Hz");

    }
    else if (stp==2) 
    {   
      step1 = 100;
   tft.fillRect(5,100,155,20,ST7735_BLACK);   //partly screen erasing by writing box
  tft.setCursor(15, 80);
  tft.setTextColor(ST7735_CYAN);
  tft.print("Step:");
  tft.setCursor(15, 105);
  tft.setTextColor(ST7735_WHITE);
  tft.print("100 Hz");
    }
    else if (stp==3) 
    {   
      step1 = 1000;
    tft.fillRect(5,100,155,20,ST7735_BLACK);   //partly screen erasing by writing box
  tft.setCursor(15, 80);
  tft.setTextColor(ST7735_CYAN);
  tft.print("Step:");
  tft.setCursor(15, 105);
  tft.setTextColor(ST7735_WHITE);
  tft.print("1 KHz");
    }
    else if (stp==4) 
    {   
      step1 = 10000;
      tft.fillRect(5,100,155,20,ST7735_BLACK);   //partly screen erasing by writing box
  tft.setCursor(15, 80);
  tft.setTextColor(ST7735_CYAN);
  tft.print("Step:");
  tft.setCursor(15, 105);
  tft.setTextColor(ST7735_WHITE);
  tft.print("10 KHz");
    }
    else if (stp==5) 
    {   
      step1 = 100000;
     tft.fillRect(5,100,155,20,ST7735_BLACK);   //partly screen erasing by writing box
  tft.setCursor(15, 80);
  tft.setTextColor(ST7735_CYAN);
  tft.print("Step:");
  tft.setCursor(15, 105);
  tft.setTextColor(ST7735_WHITE);
  tft.print("100 KHz");
    }
    else if (stp==6) 
    {   
      step1 = 1000000;
      tft.fillRect(5,100,155,20,ST7735_BLACK);   //partly screen erasing by writing box
  tft.setCursor(15, 80);
  tft.setTextColor(ST7735_CYAN);
  tft.print("Step:");
  tft.setCursor(15, 105);
  tft.setTextColor(ST7735_WHITE);
  tft.print("1 MHz");
    }
    delay(500); 
    stp_old = stp;  
  }
}

int read_Step()
{
  if (buttonState == LOW) return stp=stp+1;
}

ISR(PCINT2_vect) {
  unsigned char result = r.process();
  if (result) {    
    if (result == DIR_CW){
      rx=rx+step1;
    }
    else {
      rx=rx-step1;
    };       
    if (rx >=40000000){
      rx=rx_old;
    }; // UPPER VFO LIMIT 40MHz
    if  (rx <= 100000){
      rx=rx_old;
    }; // LOWER VFO LIMIT 100KHz
  }
}

